/*

Arquivo:

   Item.hpp

Descrio:

   Interface para um elemento da tabela de smbolos. A classe Item no possui
 nada especfico da tabela de smbolos. A classe STItem  quem faz a relao
 entre um Item e a tabela de smbolos, possuindo funes de comparao.

Excees:

   -> AllocationError         : Lanada quando h erros de alocao de memria
   -> EmptyKey                : Lanada quando se tenta criar um item de chave
                                vazia
*/

/* Especfico do gcc */
#pragma interface
/* Fim do especfico */

//
// Interface - Item
//
class Item {
  char* word;
  unsigned long freq;

public:
  /* Constantes */
  enum mode { copy_string, copy_pointer };

  /* Construtores e Destrutores */
  Item(char*, mode = copy_pointer, unsigned long = 1);
  ~Item() { delete[] word; }

  /* Mtodos */
  const char* GETword() const { return word; }
  void INCfreq() { freq++; }
  unsigned long GETfreq() const { return freq; }

  /* Classe que faz a relao entre um item e a tabela de smbolos */
  friend class STItem;

  /* Excees */
  class AllocationError { };
  class EmptyKey { };
};

//
// Interface - STItem
//
class STItem {
  static int (*cmp_func)(const Item&, const Item&);

  /* Funes Privadas */
  static int ITEMcompare_word(const Item&, const Item&);
  static int ITEMcompare_freq(const Item&, const Item&);

public:
  /* Constantes */
  enum { greater, equal, lesser };
  enum cmp_mode { compare_word, compare_freq };

  /* Mtodos para comparao com chave e hashing */
  static bool KEYcompare(const Item&, const char*);
  static unsigned long KEYhash(const char*, unsigned long);
  static unsigned long KEYhash(const Item& obj, unsigned long m)
    { return KEYhash(obj.word, m); }

  /* Mtodos para comparao entre elementos */
  static void SETcompare_mode(cmp_mode = compare_word);
  static int ITEMcompare(const Item& obj1, const Item& obj2)
    { return (*cmp_func)(obj1, obj2); }
};
