/*

Arquivo:

   ST.hpp

Descrio:

   Interface da tabela de smbolos. Qualquer tipo de elemento pode ser colocado
 na tabela de smbolos, contanto que seja tambm definida uma classe de compa-
 rao, que deve exportar trs funes: KEYcompare, KEYhash e ITEMcompare. 
 Veja o arquivo Item.hpp para um exemplo de tal classe.

Excees:

   --> AllocationError        : lanada quando h erro de alocao
   --> DoubleTableSizeError   : lanada quando ocorrem erros durante o processo
                                de dobrar o tamanho da tabela
   --> ArrayNotBuilt          : lanada quando se tenta ordenar a tabela ou 
                                selecionar um elemento com o operador [] sem 
                                que a tabela tenha antes sido transformada num
                                array

*/

/* Especfico do gcc */
#pragma interface
/* Fim do especfico */

//
// Defines
//
#define NULL_ITEM             NULL

//
// Interface
//

template<class IT, class KEY, class CMP>
class STable {
  int table_size;
  unsigned long item_count;

  struct Node {
    IT* data;
    Node* next;
  }** hash_table;

  IT** sorted_table;

  /* Funes */
  void DoubleTableSize();

public:
  /* Construtores e Destrutores */
  STable();
  ~STable();

  /* Mtodos */
  void insert(IT*);
  IT* search(const KEY&) const;
  void build_array();
  void sort(unsigned long = 0, unsigned long = 0);
  unsigned long count() const { return item_count; }

  /* Operadores */
  IT& operator[](unsigned long) const;

  /* Excees */
  class AllocationError { };
  class DoubleTableSizeError { };
  class ArrayNotBuilt { };
};
