/*

Arquivo:

   input.cpp

Descrio:

   Definio das funes encontradas no arquivo input.hpp.

*/

//
// Cabealhos
//
#include <stdio.h>
#include "input.hpp"

using Input::AllocationError;

//
// Variveis com Linkedio Interna
//
static bool past_eof = false;      // Ligado se passou do final do arquivo
static char* buffer = NULL;        // Buffer de leitura
static int buffer_size = 10;       // Tamanho inicial do buffer de leitura

//
// Prottipos
//
inline bool ValidChar(char);

//
// Funes
//

/*
 * ValidChar
 *   Retorna true se um caractere  vlido, false caso contrrio. Um caractere
 * vlido  um tal que 'A' <= c <= 'Z' ou 'a' <= c <= 'z' ou uma das variaes
 * acentuadas.
 *
 */
inline bool ValidChar(int c)
{
  return (('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z') ||
	  (192 <= c && c <= 214) || (216 <= c && c <= 246) ||
	  (248 <= c && c <= 255));
}

/*
 * InitInput
 *   Inicializa o buffer de leitura.
 *
 * Excees:
 *   Causa a exceo AllocationError se falhar em alocar memria para o buffer.
 *
 */
void InitInput()
{
  if(buffer != NULL)
    return;

  buffer = new char[buffer_size];

  if(buffer == NULL)
    throw AllocationError();
}

/*
 * DeInitInput
 *   Destri o buffer de leitura e reseta o tamanho da entrada.
 *
 */
void DeInitInput()
{
  delete[] buffer;
  buffer_size = 10;
  past_eof = false;
}

/*
 * GetWord
 *   L uma palavra da entrada padro. Uma palavra  uma sequncia maximal de
 * letras.
 *
 * Excees:
 *   Causa a exceo AllocationError se falhar em alocar espao em memria.
 *
 * Retornos:
 *   Retorna um ponteiro para o buffer no qual se encontra a string. Se nenhuma 
 * string foi lida, retorna NULL_WORD. Ao detectar o final do arquivo, aciona o
 * flag past_eof para retornar NULL_WORD na prxima tentativa de leitura.
 * 
 */
char* GetWord()
{
  int c;                        // Caractere lido

  /* Se o final do arquivo j foi atingido, retornar NULL_WORD */
  if(past_eof)
    return NULL_WORD;

  /* L at um caractere vlido */
  for(c = getchar(); c != EOF && !ValidChar(c); c = getchar());

  /* Se o ltimo caractere foi EOF, retornar NULL_WORD */
  if(c == EOF) {
    past_eof = true;
    return NULL_WORD;
  }

  /* L a palavra */
  int i = 0;
  for(int s = buffer_size - 1; ValidChar(c); c = getchar(), i++) {
    /* Verifica se  necessrio dobrar o array */
    if(i == s) {
      buffer_size *= 2;
      s = buffer_size - 1;

      char* new_buffer = new char[buffer_size];

      if(new_buffer == NULL)
	throw AllocationError();

      for(int j = 0; j < i; new_buffer[j] = buffer[j++]);

      delete[] buffer;
      buffer = new_buffer;
    }

    buffer[i] = c;
  }
 
  /* Adiciona caractere de final de string */
  buffer[i] = '\0';

  /* Se o ltimo caractere for EOF, aciona past_eof */
  if(c == EOF)
    past_eof = true;

  return buffer;
}
