/*

Arquivo:

   main.cpp

Descrio:

   Mdulo principal.

*/

//
// Cabealhos
//
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "mem.hpp"
#include "input.hpp"
#include "Item.hpp"
#include "ST.hpp"

//
// Cdigos de erro
//
#define SUCCESS               0
#define ERROR                 1

//
// Typedefs
//
typedef STable<Item, char*, STItem> STableItem;

//
// Prottipos
//
static int DoWordCount(unsigned long);

//
// Main
//
int main(int argc, char* argv[])
{
  unsigned long n = 0;        // Nmero de palavras da sada

  /* Processa linha de comando */
  if(argc < 2) {
    cout << "Poucos argumentos." << endl;

    return ERROR;
  }
  else if(argc > 2) {
    cout << "Muitos argumentos." << endl;

    return ERROR;
  }
  else if(strncmp(argv[1], "-n", 2) != 0) {
    cout << "Argumento " << argv[1] << " invlido." << endl;

    return ERROR;
  }

  int arg_len = strlen(argv[1]) + 2;
  for(int i = 2; i < arg_len; i++)
    if(argv[1][i] == '-') {
      cout << "Nmero invlido." << endl;

      return ERROR;
    }

  char* invalid_char;
  n = strtoul(argv[1] + 2, &invalid_char, 10);

  if((*invalid_char == '\0' && *(argv[1] + 2) == '\0') ||
     (*invalid_char != '\0')) {
    cout << "Nmero invlido." << endl;

    return ERROR;
  }

  /* Inicializa funes de entrada */
  try {
    InitInput();
  }
  catch(Input::AllocationError) {
    cout << "Erro de alocao de memria ao inicializar funes de leitura de "
	 << "entrada." << endl;

    return ERROR;
  }

  /* Resolve o Problema */
  int ret = DoWordCount(n);

  /* Encerra o Programa */
  DeInitInput();

  return ret;
}

//
// Funes
//
static int DoWordCount(unsigned long n)
{
  char* word;

  try {
    double solving_time = clock();

    STableItem symbol_table;

    while((word = GetWord()) != NULL_WORD) {
      Item* ret = symbol_table.search(word);
      
      if(ret != NULL)
	ret->INCfreq();
      else {
	mem_ptr<Item> new_item = new Item(word, Item::copy_string);
	symbol_table.insert(new_item);
	new_item.release();
      }
    }

    /* Ordena a lista de palavras por frequncia e em seguida ordena os trechos
     * de mesma frequncia por ordem alfabtica
     */
    symbol_table.build_array();
    STItem::SETcompare_mode(STItem::compare_freq);
    symbol_table.sort();
    STItem::SETcompare_mode(STItem::compare_word);

    unsigned long i, j;
    unsigned long num_items = symbol_table.count();
    for(i = 0; i < num_items; i = j) {
      for(j = i+1; j < num_items && 
	    symbol_table[j].GETfreq() == symbol_table[j-1].GETfreq(); j++);

      symbol_table.sort(i, j-i);
    }

    /* Tempo de resoluo */
    solving_time = clock() - solving_time;
    solving_time /= CLOCKS_PER_SEC;

    if(symbol_table.count() < n) {
      cout << "H menos palavras distintas do que foi pedido para a exibio."
	   << endl << "Todas sero exibidas." << endl << endl;

      n = symbol_table.count();
    }

    for(unsigned long i = 0; i < n; i++)
      cout << symbol_table[i].GETfreq() << "  "
	   << symbol_table[i].GETword() << endl;

    cout << endl << "Total de palavras distintas: " << num_items << endl;
    cout << "Tempo para resoluo: " << solving_time << " segundos." << endl;
  }
  catch(STableItem::AllocationError) {
    cout << "Erro de alocao de memria causado pela tabela de smbolos." 
	 << endl;

    return ERROR;
  }
  catch(STableItem::DoubleTableSizeError) {
    cout << "Erro ao tentar dobrar o tamanho da tabela de hash." << endl;

    return ERROR;
  }
  catch(STableItem::ArrayNotBuilt) {
    cout << "Tentativa de ordenao/iterao sem construo de array." << endl;

    return ERROR;
  }
  catch(Item::AllocationError) {
    cout << "Erro de alocao de memria causado pelo item." << endl;

    return ERROR;
  }
  catch(Item::EmptyKey) {
    cout << "Tentativa de criar/manipular item com chave nula." << endl;
   
    return ERROR;
  }
  catch(Input::AllocationError) {
    cout << "Erro de alocao de memria para a leitura da entrada." << endl;

    return ERROR;
  }
  catch(mem_ptr<Item>::AllocationError) {
    cout << "Erro de alocao de memria para insero de novo elemento." 
	 << endl;

    return ERROR;
  }

  return SUCCESS;
}
